#ifndef _CARDAPPZ_DESFIRE_H_
#define _CARDAPPZ_DESFIRE_H_

#include "cardappz_types.h"
#include "cardappz_errors.h"
#include "cardappz_transport.h"

#include "des.h"

/*
 * Corporate ID application
 * ------------------------
 */
#define CORPID_DESFIRE_AID 0x504101
#define CORPID_DESFIRE_FID_IDENTITY 1
#define CORPID_DESFIRE_FID_FINGERPRINTS 2
#define CORPID_DESFIRE_FID_PICTURE 3
#define CORPID_DESFIRE_FID_ACCESSCONTROL 4
#define CORPID_DESFIRE_FID_OPTIONSSIGN 5

/* Read-only mode */
/* -------------- */

/* Select application, authentication with key 1 */
int DesfireSelectCorpid(CARDAPPZ_CTX_ST *ctx);

/* Read different files */
int DesfireReadCorpidIdentity(CARDAPPZ_CTX_ST *ctx, CORPID_IDENTITY_ST *identity);
int DesfireReadCorpidFingerprints(CARDAPPZ_CTX_ST *ctx, unsigned char fingerprints[CORPID_FINGERPRINTS_MAX_SZ]);
int DesfireReadCorpidPicture(CARDAPPZ_CTX_ST *ctx, unsigned char picture[CORPID_PICTURE_MAX_SZ]);
int DesfireReadCorpidAccessControl(CARDAPPZ_CTX_ST *ctx, unsigned char accesscontrol[48]);
int DesfireReadCorpidOptionsSign(CARDAPPZ_CTX_ST *ctx, CORPID_OPTIONSSIGN_ST *optionssign);

/* Personalisation */
/* --------------- */

int DesfireWriteCorpid(CARDAPPZ_CTX_ST *ctx, CORPID_IDENTITY_ST *identity, unsigned char fingerprints[CORPID_FINGERPRINTS_MAX_SZ], unsigned char picture[CORPID_PICTURE_MAX_SZ], unsigned char accesscontrol[48], CORPID_OPTIONSSIGN_ST *optionssign);

/* Formatting */
/* ---------- */
int DesfireFormatCorpid(CARDAPPZ_CTX_ST *ctx);

/*
 * Electronic purse application
 * ----------------------------
 */
#define PURSE_DESFIRE_AID 0x504102
#define PURSE_DESFIRE_FID_BALANCE 1
#define PURSE_DESFIRE_FID_RELOAD 2
#define PURSE_DESFIRE_FID_PURCHASE 3

/* Read and decrement mode */
/* ----------------------- */

/* Select application, authentication with key 1 */
int DesfireSelectPurse(CARDAPPZ_CTX_ST *ctx);

/* Read different files */
int DesfireReadPurseBalance(CARDAPPZ_CTX_ST *ctx, unsigned long *Balance);
int DesfireReadPurseReloadEvent(CARDAPPZ_CTX_ST *ctx, int Index, PURSE_RELOAD_ST *ReloadEntry);
int DesfireReadPursePurchaseEvent(CARDAPPZ_CTX_ST *ctx, int Index, PURSE_PURCHASE_ST *PurchaseEntry);

/* New purchase */
int DesfireDebitPurse(CARDAPPZ_CTX_ST *ctx, PURSE_PURCHASE_ST *PurchaseEntry);

/* Reloading */
/* --------- */

int DesfireCreditPurse(CARDAPPZ_CTX_ST *ctx, PURSE_RELOAD_ST *ReloadEntry);

/* Formatting */
/* ---------- */

int DesfireFormatPurse(CARDAPPZ_CTX_ST *ctx);

/*
 * Public transportation application
 * ---------------------------------
 */
#define TRANSPORT_DESFIRE_AID 0x504103
#define TRANSPORT_DESFIRE_FID_CONTRACTS 1
#define TRANSPORT_DESFIRE_FID_EVENTS 2
#define TRANSPORT_DESFIRE_FID_COUNTER1 3
#define TRANSPORT_DESFIRE_FID_COUNTER2 4
#define TRANSPORT_DESFIRE_FID_COUNTER3 5

/* Read and decrement mode */
/* ----------------------- */

/* Select application, authentication with key 1 */
int DesfireSelectTransport(CARDAPPZ_CTX_ST *ctx);

/* Read different files */
int DesfireReadTransportContract(CARDAPPZ_CTX_ST *ctx, int Index, TRANSPORT_CONTRACT_ST *Contract);
int DesfireReadTransportCounter(CARDAPPZ_CTX_ST *ctx, int Index, unsigned long *Value);
int DesfireReadTransportEvent(CARDAPPZ_CTX_ST *ctx, int Index, TRANSPORT_EVENT_ST *Event);

/* New event + decrement counter if needed */
int DesfireAppendTransportEvent(CARDAPPZ_CTX_ST *ctx, TRANSPORT_EVENT_ST *Event);

/* Contract writing */
/* ---------------- */

int DesfireAppendTransportContract(CARDAPPZ_CTX_ST *ctx, TRANSPORT_CONTRACT_ST *Contract, unsigned long CounterValue);
int DesfireEraseTransportContracts(CARDAPPZ_CTX_ST *ctx);

/* Formatting */
/* ---------- */

int DesfireFormatTransport(CARDAPPZ_CTX_ST *ctx);

/*
 * Common part
 * -----------
 */
#define DESFIRE_COMM_PLAIN  0
#define DESFIRE_COMM_MACED  1
#define DESFIRE_COMM_CIPHER 3
 
static const unsigned char DESFIRE_BLANK_KEY[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

void TDES_Decrypt_CBC_S(TDES_CTX_ST *tdes_ctx, unsigned char *data, int blocks);
void TDES_Decrypt_CBC_R(TDES_CTX_ST *tdes_ctx, unsigned char *data, int blocks);
void TDES_Encrypt_CBC_S(TDES_CTX_ST *tdes_ctx, unsigned char *data, int blocks);

int DesfireAuthenticate(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, const unsigned char KeyValue[16]);
int DesfireAuthenticateWithSam(CARDAPPZ_CTX_ST *ctx, unsigned char PiccKeyIndex, unsigned char SamKeyIndex, unsigned char SamKeyVersion);

int DesfireChangeKey(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, const unsigned char KeyValue[16]);
int DesfireCreateApplication(CARDAPPZ_CTX_ST *ctx, unsigned long Aid, unsigned char KeySettings, unsigned char KeyCount);
int DesfireSelectApplication(CARDAPPZ_CTX_ST *ctx, unsigned long Aid);
int DesfireSelectCard(CARDAPPZ_CTX_ST *ctx);
int DesfireFormatPICC(CARDAPPZ_CTX_ST *ctx);
int DesfireCreateStdDataFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommSettings, unsigned short AccessRights, unsigned long FileSize);
int DesfireCreateValueFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommSettings, unsigned short AccessRights, signed long LowerLimit, signed long UpperLimit, signed long CurrentValue, unsigned char LimitedCredit);
int DesfireCreateLinearRecordFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommSettings, unsigned short AccessRights, unsigned long RecordSize, unsigned long RecordCount);
int DesfireCreateCyclicRecordFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommSettings, unsigned short AccessRights, unsigned long RecordSize, unsigned long RecordCount);
int DesfireReadData(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, unsigned long Offset, unsigned long Length, unsigned char Buffer[], unsigned long *ActualLength);
int DesfireWriteData(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, unsigned long Offset, unsigned long Length, const unsigned char Buffer[]);
int DesfireGetValue(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, signed long *Value);
int DesfireCredit(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, signed long Value);
int DesfireDebit(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, signed long Value);
int DesfireWriteRecord(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, unsigned long Offset, unsigned long Length, const unsigned char Buffer[]);
int DesfireReadRecords(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex, unsigned char CommMode, unsigned long Offset, unsigned long Length, unsigned char Buffer[], unsigned long *ActualLength);
int DesfireClearRecordFile(CARDAPPZ_CTX_ST *ctx, unsigned char FileIndex);
int DesfireCommitTransaction(CARDAPPZ_CTX_ST *ctx);
int DesfireAbortTransaction(CARDAPPZ_CTX_ST *ctx);

int DesfireSelectCard(CARDAPPZ_CTX_ST *ctx);
int DesfireBlankCard(CARDAPPZ_CTX_ST *ctx);




int DesfireSamMenu(CARDAPPZ_CTX_ST *ctx);


int DesfireSAMAuthenticatePICC_1(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, unsigned char KeyVersion, unsigned char PiccChallenge[8], unsigned char SamCryptogram[16]);
int DesfireSAMAuthenticatePICC_2(CARDAPPZ_CTX_ST *ctx, unsigned char PiccCryptogram[8]);

int DesfireSAMVerifyMAC(CARDAPPZ_CTX_ST *ctx, const unsigned char Buffer[], unsigned long Length);
int DesfireSAMGenerateMAC(CARDAPPZ_CTX_ST *ctx, const unsigned char Buffer[], unsigned long Length, unsigned char Mac[4]);

int DesfireSAMEncipherData(CARDAPPZ_CTX_ST *ctx, unsigned char Buffer[], unsigned long InLength, unsigned long *OutLength);
int DesfireSAMDecipherData(CARDAPPZ_CTX_ST *ctx, unsigned char Buffer[], unsigned long InLength, unsigned long *OutLength);

int DesfireSAMSelectKey(CARDAPPZ_CTX_ST *ctx, unsigned char KeyIndex, unsigned char KeyVersion);


#endif
