unit uPurse;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, IniFiles,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, Grids, uSpringProxDll, uSpringProxMif,
  jpeg;

type
  TfPurse = class(TForm)
    status: TStatusBar;
    gbIdentity: TGroupBox;
    lbLName: TLabel;
    eLName: TEdit;
    eFName: TEdit;
    lbFName: TLabel;
    Label1: TLabel;
    lbOther: TLabel;
    rbMale: TRadioButton;
    rbFemale: TRadioButton;
    mOther: TMemo;
    btnIdentityUpdate: TButton;
    gbPurchase: TGroupBox;
    Label2: TLabel;
    eCurrentValue: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    eInitialValue: TEdit;
    eDateReload: TEdit;
    Label5: TLabel;
    eDateExpire: TEdit;
    Label6: TLabel;
    ePurchase: TEdit;
    btnPay: TButton;
    gbReload: TGroupBox;
    Label7: TLabel;
    cbReloadAmount: TComboBox;
    Label8: TLabel;
    cbDuration: TComboBox;
    btnReload: TButton;
    eBirthDate: TEdit;
    pTop: TPanel;
    lbTitle: TLabel;
    lbCaption: TLabel;
    pLeft: TPanel;
    img14443: TImage;
    imgLogo: TImage;
    btnReaderOpen: TButton;
    btnReaderClose: TButton;
    btnTagSelect: TButton;
    imgHeader: TImage;
    procedure ClearInfos;
    procedure ClearPurse;
    procedure btnReaderOpenClick(Sender: TObject);
    procedure btnReaderCloseClick(Sender: TObject);
    procedure btnTagSelectClick(Sender: TObject);
    procedure btnIdentityUpdateClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    function ReadInfos : boolean;
    function WriteInfos : boolean;
    function ReadPurse : boolean;
    function WritePurse : boolean;
    function Purchase : boolean;
    procedure FormCreate(Sender: TObject);
    procedure IdentityChanged(Sender: TObject);
    procedure btnReloadClick(Sender: TObject);
    procedure ePurchaseChange(Sender: TObject);
    procedure btnPayClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    tag_info : TMifStTagInfo;
    tag_mad  : TMifStMadSector;
    sect_info_1, sect_info_2, sect_purse : byte;
  public
    { Dclarations publiques }
  end;

var
  fPurse: TfPurse;

implementation

{$R *.dfm}

uses uSpringProxAid;

function StringToAmount(s : string) : longint;
var
   t : string;
   m, f : longint;
begin
   // Make sure we have the format %d.%02d
   if Pos('.', s) > 0 then
   begin
      t := Copy(s, 1, Pos('.', s)-1);
      m := StrToIntDef(t, 0);
      t := Copy(Copy(s, Pos('.', s)+1, 2) + '00', 1, 2);
      f := StrToIntDef(t, 0);
   end else
   if Pos(',', s) > 0 then
   begin
      t := Copy(s, 1, Pos(',', s)-1);
      m := StrToIntDef(t, 0);
      t := Copy(Copy(s, Pos(',', s)+1, 2) + '00', 1, 2);
      f := StrToIntDef(t, 0);
   end else
   begin
      m := StrToIntDef(s, 0);
      f := 0;
   end;
   result := (100 * m) + f;
end;

procedure TfPurse.FormCreate(Sender: TObject);
var
   ini : TIniFile;
   pic : TPicture;
   str_company : string;
   str_logo    : string;
   str_product : string;
begin
   ini := TIniFile.Create(ExtractFileDir(Application.ExeName) + '\csbinfo.ini');
   str_company := ini.ReadString('company', 'name', 'SpringCard');
   str_logo    := ini.ReadString('company', 'logo', 'logo_pro-active.bmp');
   str_product := ini.ReadString('product', 'name', 'CSB / SpringProx');
   ini.Free;
   pic := TPicture.Create;
   try
      pic.LoadFromFile(str_logo);
      imgLogo.Left := pTop.Width - 8 - pic.Width;
      imgLogo.Picture.Assign(pic);
   except
      // No picture !
   end;
   pic.Free;
   Caption := str_company + ' ' + str_product + ' - MfPurse Sample Project';
end;

procedure TfPurse.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   SPROX_ReaderClose;
end;

procedure TfPurse.ClearInfos;
begin
   eLName.Text := '';
   eFName.Text := '';
   eBirthDate.Text := '';
   rbMale.Checked := false;
   rbFemale.Checked := false;
   mOther.Lines.Clear;
   btnIdentityUpdate.Enabled := false;
   gbIdentity.Enabled := false;
   Application.ProcessMessages;
end;

procedure TfPurse.ClearPurse;
begin
   eCurrentValue.Text := '';
   eInitialValue.Text := '';
   eDateReload.Text := '';
   eDateExpire.Text := '';
   gbPurchase.Enabled := false;
   gbReload.Enabled := false;
   btnPay.Enabled := false;
   btnReload.Enabled := false;
   Application.ProcessMessages;
end;

procedure TfPurse.IdentityChanged(Sender: TObject);
begin
   if gbIdentity.Enabled then btnIdentityUpdate.Enabled := true;
end;

procedure TfPurse.ePurchaseChange(Sender: TObject);
var
   p : longint;
   v : longint;
begin
   v := StringToAmount(eCurrentValue.Text);
   p := StringToAmount(ePurchase.Text);
   btnPay.Enabled := (v > p);
end;

procedure TfPurse.btnPayClick(Sender: TObject);
var
   p : longint;
   v : longint;
begin
   btnPay.Enabled := false;
   v := StringToAmount(eCurrentValue.Text);
   p := StringToAmount(ePurchase.Text);
   ePurchase.Text := Format('%d.%0.2d', [p div 100, p mod 100]);
   Application.ProcessMessages;
   if v > p then
   begin
      if not Purchase then
      begin
         exit;
      end;
      p := 0;
      ePurchase.Text := Format('%d.%0.2d', [p div 100, p mod 100]);
      status.SimpleText := 'Done';
   end;
end;

procedure TfPurse.btnReaderOpenClick(Sender: TObject);
var
   rc : sword;
begin
   status.SimpleText := '';
   btnReaderOpen.Enabled := false;
   // Open the reader
   rc := SPROX_ReaderOpen(nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Reader not found !', []),
                 mtWarning,
                 [mbOk],
                 0);
      btnReaderOpen.Enabled := true;
      exit;
   end;
   // Reader is opened
   btnReaderClose.Enabled := true;
   // Load usefull keys
                  rc := SPROX_MifLoadKey(false, 'A', 0, @key_aaaaaa);
   if rc = 0 then rc := SPROX_MifLoadKey(false, 'A', 1, @key_demo_a);
   if rc = 0 then rc := SPROX_MifLoadKey(false, 'A', 2, @key_test_a);
   if rc = 0 then rc := SPROX_MifLoadKey(false, 'A', 3, @key_ffffff);
   if rc = 0 then rc := SPROX_MifLoadKey(false, 'B', 0, @key_bbbbbb);
   if rc = 0 then rc := SPROX_MifLoadKey(false, 'B', 1, @key_demo_b);
   if rc = 0 then rc := SPROX_MifLoadKey(false, 'B', 2, @key_test_b);
   if rc = 0 then rc := SPROX_MifLoadKey(false, 'B', 3, @key_ffffff);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to load Mifare keys into the reader !', []),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   // Reader is ready
   status.SimpleText := 'Ready';
   btnTagSelect.Enabled := true;
end;

procedure TfPurse.btnReaderCloseClick(Sender: TObject);
var
   rc : sword;
begin
   btnReaderClose.Enabled := false;
   btnTagSelect.Enabled := false;
   status.SimpleText := '';
   ClearInfos;
   rc := SPROX_ReaderClose;
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
   end else
   begin
      status.SimpleText := 'Closed';
   end;
   btnReaderOpen.Enabled := true;
end;

procedure TfPurse.btnIdentityUpdateClick(Sender: TObject);
begin
   btnIdentityUpdate.Enabled := false;
   status.SimpleText := '';
   if (not WriteInfos) or (not ReadInfos) then
   begin
      gbIdentity.Enabled := true;
      btnIdentityUpdate.Enabled := true;
      exit;
   end;
   gbIdentity.Enabled := true;
   status.SimpleText := 'Done';
end;

procedure TfPurse.btnReloadClick(Sender: TObject);
begin
   btnReload.Enabled := false;
   status.SimpleText := '';
   if (not WritePurse) or (not ReadPurse) then
   begin
      gbPurchase.Enabled := true;
      gbReload.Enabled := true;
      if StringToAmount(eCurrentValue.Text) < 10000 then btnReload.Enabled := true;
      exit;
   end;
   gbPurchase.Enabled := true;
   gbReload.Enabled := true;
   if StringToAmount(eCurrentValue.Text) < 10000 then btnReload.Enabled := true;
   status.SimpleText := 'Done';
end;

procedure TfPurse.btnTagSelectClick(Sender: TObject);
var
   buffer : TMifStSector;
   b, o : byte;
   rc : sword;
begin
   btnTagSelect.Enabled := false;
   ClearInfos;
   status.SimpleText := '';
   // Select one tag
   rc := SPROX_MifStSelectAny(@(tag_info.snr), @(tag_info.atq), @(tag_info.ats));
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      btnTagSelect.Enabled := true;
      exit;
   end;
   // Read tag's MAD (sector 0)
   rc := SPROX_MifStReadSector(@(tag_info.snr), 0, @tag_mad, nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to read the MAD', []),
                 mtWarning,
                 [mbOk],
                 0);
      btnTagSelect.Enabled := true;
      exit;
   end;
   // Find the info sectors
   if not MifStGetAidSector(tag_mad, aid_info_1, sect_info_1) then sect_info_1 := 0;
   if not MifStGetAidSector(tag_mad, aid_info_2, sect_info_2) then sect_info_2 := 0;
   if not MifStGetAidSector(tag_mad, aid_purse, sect_purse) then sect_purse := 0;
   // Is it OK ?
   if (sect_info_1 = 0) or (sect_info_2 = 0) or (sect_purse = 0) then
   begin
      if MessageDlg('The supplied tag is not formatted as required. To you want to format it now ?',
                    mtWarning,
                    [mbYes, mbNo],
                    0) <> mrYes then
      begin
         status.SimpleText := 'Tag not formatted';
         btnTagSelect.Enabled := true;
         exit;
      end;
      for b := 0 to 2 do for o := 0 to 15 do buffer[b][o] := $00;
      // Let's try to format the tag
      if sect_info_1 = 0 then
      begin
         // Find room for info 1
         if not MifStGetFreeSector(tag_mad, sect_info_1) then
         begin
            status.SimpleText := 'No available sector on the tag !';
            btnTagSelect.Enabled := true;
            exit;
         end;
         MifStSetSectorAid(tag_mad, sect_info_1, aid_info_1);
         // Format the selected sector
         rc := SPROX_MifStUpdateAccessBlock(nil,
                                            sect_info_1,
                                            nil,
                                            @key_demo_a, // Only our applications can read it
                                            @key_demo_b, // Only our applications can write it
                                            acc_normal,
                                            acc_normal,
                                            acc_normal,
                                            acc_security);
         if rc <> 0 then
         begin
            status.SimpleText := SPROX_GetErrorMessage(rc);
            MessageDlg(Format('Failed to format sector %d (identity part 1)', [sect_info_1]),
                       mtWarning,
                       [mbOk],
                       0);
            btnTagSelect.Enabled := true;
            exit;
         end;
         // "Clean" the selected sector
         rc := SPROX_MifStWriteSector(nil, sect_info_1, @buffer, nil);
         if rc <> 0 then
         begin
            status.SimpleText := SPROX_GetErrorMessage(rc);
            MessageDlg(Format('Failed to clean sector %d (identity part 1)', [sect_info_1]),
                       mtWarning,
                       [mbOk],
                       0);
            btnTagSelect.Enabled := true;
            exit;
         end;
      end;
      if sect_info_2 = 0 then
      begin
         // Find room for info 2
         if not MifStGetFreeSector(tag_mad, sect_info_2) then
         begin
            status.SimpleText := 'No available sector on the tag !';
            btnTagSelect.Enabled := true;
            exit;
         end;
         MifStSetSectorAid(tag_mad, sect_info_2, aid_info_2);
         // Format the selected sector
         rc := SPROX_MifStUpdateAccessBlock(nil,
                                            sect_info_2,
                                            nil,
                                            @key_demo_a, // Only our applications can read it
                                            @key_demo_b, // Only our applications can write it
                                            acc_normal,
                                            acc_normal,
                                            acc_normal,
                                            acc_security);
         if rc <> 0 then
         begin
            status.SimpleText := SPROX_GetErrorMessage(rc);
            MessageDlg(Format('Failed to format sector %d (identity part 2)', [sect_info_2]),
                       mtWarning,
                       [mbOk],
                       0);
            btnTagSelect.Enabled := true;
            exit;
         end;
         // "Clean" the selected sector
         rc := SPROX_MifStWriteSector(nil, sect_info_2, @buffer, nil);
         if rc <> 0 then
         begin
            status.SimpleText := SPROX_GetErrorMessage(rc);
            MessageDlg(Format('Failed to clean sector %d (identity part 2)', [sect_info_2]),
                       mtWarning,
                       [mbOk],
                       0);
            btnTagSelect.Enabled := true;
            exit;
         end;
      end;
      if sect_purse = 0 then
      begin
         // Find room for purse
         if not MifStGetFreeSector(tag_mad, sect_purse) then
         begin
            status.SimpleText := 'No available sector on the tag !';
            btnTagSelect.Enabled := true;
            exit;
         end;
         MifStSetSectorAid(tag_mad, sect_purse, aid_purse);
         // Format the selected sector
         rc := SPROX_MifStUpdateAccessBlock(nil,
                                            sect_purse,
                                            nil,
                                            @key_demo_a, // Only our applications can read it
                                            @key_demo_b, // Only our applications can write it
                                            acc_counter, // the counter block !!!
                                            acc_normal,
                                            acc_normal,
                                            acc_security);
         if rc <> 0 then
         begin
            status.SimpleText := SPROX_GetErrorMessage(rc);
            MessageDlg(Format('Failed to format sector %d (purse storage)', [sect_purse]),
                       mtWarning,
                       [mbOk],
                       0);
            btnTagSelect.Enabled := true;
            exit;
         end;
         // "Clean" the selected sector
         rc := SPROX_MifStWriteSector(nil, sect_purse, @buffer, nil);
         if rc <> 0 then
         begin
            status.SimpleText := SPROX_GetErrorMessage(rc);
            MessageDlg(Format('Failed to empty sector %d (purse storage)', [sect_purse]),
                       mtWarning,
                       [mbOk],
                       0);
            btnTagSelect.Enabled := true;
            exit;
         end;
      end;
      // The MAD has been altered, we have to rewrite it
      rc := SPROX_MifStWriteSector(nil, 0, @tag_mad, nil);
      if rc <> 0 then
      begin
         status.SimpleText := SPROX_GetErrorMessage(rc);
         MessageDlg(Format('Failed to update the MAD', []),
                    mtWarning,
                    [mbOk],
                    0);
         btnTagSelect.Enabled := true;
         exit;
      end;
      // BTW, make sure access keys for the MAD are OK
      rc := SPROX_MifStUpdateAccessBlock(nil,
                                         0,
                                         nil,
                                         @key_aaaaaa, // Everybody can read it
                                         @key_demo_b, // Only our applications can write it
                                         acc_normal,
                                         acc_normal,
                                         acc_normal,
                                         acc_security);
      if rc <> 0 then
      begin
         status.SimpleText := SPROX_GetErrorMessage(rc);
         MessageDlg(Format('Failed to format the MAD', []),
                    mtWarning,
                    [mbOk],
                    0);
         btnTagSelect.Enabled := true;
         exit;
      end;
   end;
   // OK, now I can read my data !
   if (not ReadInfos) or (not ReadPurse) then
   begin
      btnTagSelect.Enabled := true;
      exit;
   end;
   // I can work with the data
   gbIdentity.Enabled := true;
   gbPurchase.Enabled := true;
   gbReload.Enabled := true;
   if StringToAmount(eCurrentValue.Text) < 10000 then btnReload.Enabled := true;
   // And I can also read another tag
   status.SimpleText := 'Ready';
   btnTagSelect.Enabled := true;
end;

function TfPurse.ReadInfos : boolean;
var
   tmp : array[1..45] of char;
   dtt : TDateTime;
   info_1 : TInfo1;
   info_2 : TInfo2;
   rc : word;
begin
   result := false;
   ClearInfos;
   // Read part 1
   rc := SPROX_MifStReadSector(nil, sect_info_1, @info_1, nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to read sector %d (identity part 1)', [sect_info_1]),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   // Read part 2
   rc := SPROX_MifStReadSector(nil, sect_info_2, @info_2, nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to read sector %d (identity part 2)', [sect_info_2]),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   // Copy data into fields
   StrLCopy(pchar(@tmp), pchar(@(info_1.lname)), 24);
   eLName.Text := StrPas(pchar(@tmp));
   StrLCopy(pchar(@tmp), pchar(@(info_1.fname)), 24);
   eFName.Text := StrPas(pchar(@tmp));
   StrLCopy(pchar(@tmp), pchar(@(info_2.other)), 44);
   mOther.Lines.Text := StrPas(pchar(@tmp));
   // This is sex
   if (info_2.sex = 'm') or (info_2.sex = 'M') then rbMale.Checked := true;
   if (info_2.sex = 'f') or (info_2.sex = 'F') then rbFemale.Checked := true;
   // This is birthdate
   if TryEncodeDate(info_2.bdate[1] + 1900, info_2.bdate[2], info_2.bdate[3], dtt) then
   begin
      eBirthDate.Text := DateToStr(dtt);
   end;
   result := true;
end;

function TfPurse.WriteInfos : boolean;
var
   tmp : array[1..45] of char;
   y, m, d : word;
   i : byte;
   info_1 : TInfo1;
   info_2 : TInfo2;
   dtt : TDateTime;
   rc : word;
begin
   result := false;
   // Copy fields into data buffers
   StrPLCopy(pchar(@tmp), eLName.Text, 24);
   for i := 0 to 24 do info_1.lname[i] := tmp[i];
   StrPLCopy(pchar(@tmp), eFName.Text, 24);
   for i := 0 to 24 do info_1.fname[i] := tmp[i];
   StrPLCopy(pchar(@tmp), mOther.Lines.Text, 44);
   for i := 0 to 44 do info_2.other[i] := tmp[i];
   // This is sex
   if rbMale.Checked then info_2.sex := 'm';
   if rbFemale.Checked then info_2.sex := 'f';
   // This is birthdate
   try
      dtt := StrToDate(eBirthDate.Text);
   except
      MessageDlg('The birthdate entered is invalid, please correct.',
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   try
      DecodeDate(dtt, y, m, d);
   except
      y := 1900;
      m := 1;
      d := 1;
   end;
   info_2.bdate[1] := byte(y mod 1900);
   info_2.bdate[2] := m;
   info_2.bdate[3] := d;
   // Write part 1
   rc := SPROX_MifStWriteSector(nil, sect_info_1, @info_1, nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to write sector %d (identity part 1)', [sect_info_1]),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   // Write part 2
   rc := SPROX_MifStWriteSector(nil, sect_info_2, @info_2, nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to write sector %d (identity part 2)', [sect_info_2]),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   result := true;
end;

function TfPurse.ReadPurse : boolean;
var
   purse : TPurse;
   value : longint;
   dtt : TDateTime;
   rc : word;
begin
   result := false;
   ClearPurse;
   // Read purse
   rc := SPROX_MifStReadSector(nil, sect_purse, @purse, nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to read sector %d (purse storage)', [sect_purse]),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   // Read the counter with the right function
   rc := SPROX_MifStReadCounter(nil, 4*sect_purse, value, nil);
   if rc <> 0 then
   begin
      // The counter may be invalid, this is not a fatal error
      MessageDlg(Format('Purse value is invalid !', []),
                 mtWarning,
                 [mbOk],
                 0);
      value := 0;
   end;
   // Display data
   eCurrentValue.Text := Format('%d.%0.2d', [value div 100, value mod 100]);
   eInitialValue.Text := Format('%d.%0.2d', [purse.value_init div 100, purse.value_init mod 100]);
   // This is the reload date
   if TryEncodeDate(purse.date_init[1] + 1900, purse.date_init[2], purse.date_init[3], dtt) then
      eDateReload.Text := DateToStr(dtt);
   // This is the expire date
   if TryEncodeDate(purse.date_expire[1] + 1900, purse.date_expire[2], purse.date_expire[3], dtt) then
      eDateExpire.Text := DateToStr(dtt);
   result := true;
end;

function TfPurse.WritePurse : boolean;
var
   purse : TPurse;
   value : longint;
   y, m, d : word;
   rc : word;
   p : PByteArray;
begin
   result := false;
   // Read the counter with the appropriate function
   rc := SPROX_MifStReadCounter(nil, 4*sect_purse, value, nil);
   if rc <> 0 then value := 0;
   // New counter value ?
   value := StringToAmount(eCurrentValue.Text) + StringToAmount(cbReloadAmount.Text);
   rc := SPROX_MifStWriteCounter(nil, 4*sect_purse, value, nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to write purse value in block 0 of sector %d (purse storage)', [sect_purse]),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   // Encode initial value
   purse.value_init := value;
   // Encode reload date
   DecodeDate(date, y, m, d);
   purse.date_init[1] := byte(y mod 1900);
   purse.date_init[2] := byte(m);
   purse.date_init[3] := byte(d);
   // Encode duration
   case cbDuration.ItemIndex of
   0 : DecodeDate(IncMonth(date, 1), y, m, d);
   1 : DecodeDate(IncMonth(date, 2), y, m, d);
   2 : DecodeDate(IncMonth(date, 4), y, m, d);
   3 : DecodeDate(IncMonth(date, 6), y, m, d);
   4 : DecodeDate(IncMonth(date, 12), y, m, d);
   else
       // Tomorrow !
       DecodeDate(date + 1, y, m, d);
   end;
   purse.date_expire[1] := byte(y mod 1900);
   purse.date_expire[2] := byte(m);
   purse.date_expire[3] := byte(d);
   // I MUST NOT alter the counter block with SPROX_MifStWriteSector, but use SPROX_MifStWriteCounter instead
   p := @purse;
   rc := SPROX_MifStWriteBlock(nil, 4*sect_purse + 1, @(p[16]), nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to write block 1 of sector %d (purse storage)', [sect_purse]),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   result := true;
end;

function TfPurse.Purchase : boolean;
var
   value : longint;
   rc : word;
begin
   result := false;
   value := StringToAmount(ePurchase.Text);
   rc := SPROX_MifStDecrementCounter(@(tag_info.snr), 4*sect_purse, value, nil);
   if rc <> 0 then
   begin
      status.SimpleText := SPROX_GetErrorMessage(rc);
      MessageDlg(Format('Failed to decrement purse value in block 0 of sector %d (purse storage)', [sect_purse]),
                 mtWarning,
                 [mbOk],
                 0);
      exit;
   end;
   // Read again
   rc := SPROX_MifStReadCounter(nil, 4*sect_purse, value, nil);
   if rc <> 0 then value := 0;
   eCurrentValue.Text := Format('%d.%0.2d', [value div 100, value mod 100]);
   if StringToAmount(eCurrentValue.Text) < 10000 then btnReload.Enabled := true;
   result := true;
end;

procedure TfPurse.FormShow(Sender: TObject);
begin
   btnReaderOpen.Enabled := true;
   btnReaderClose.Enabled := false;
   btnTagSelect.Enabled := false;
   ClearInfos;
   ClearPurse;
end;

end.
