unit uShowUid;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, IniFiles, Grids, jpeg;

type
  TfShowUid = class(TForm)
    status: TStatusBar;
    pTop: TPanel;
    lbTitle: TLabel;
    lbCaption: TLabel;
    imgLogo: TImage;
    pLeft: TPanel;
    img14443: TImage;
    btnReaderOpen: TButton;
    btnReaderClose: TButton;
    sgTags: TStringGrid;
    cbAPolling: TCheckBox;
    tmrPoll: TTimer;
    cbBPolling: TCheckBox;
    imgHeader: TImage;
    procedure btnReaderOpenClick(Sender: TObject);
    procedure btnReaderCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure tmrPollTimer(Sender: TObject);
  private
    toggle : boolean;
  public

  end;

var
  fShowUid: TfShowUid;

implementation

{$R *.dfm}

uses uSpringProxDll;

procedure TfShowUid.btnReaderOpenClick(Sender: TObject);
var
   rc     : sword;
   buffer : array[1..128] of char;
begin
   Screen.Cursor := crHourglass;
   btnReaderOpen.Enabled := false;
   //
   // Try to open the reader
   // ----------------------
   // Setting device to NULL leads to the automatic discovery procedure
   //
   rc := SPROX_ReaderOpen(nil);
   if rc = 0 then
   begin
      // Success

      // Get reader device name
      // ----------------------

      rc := SPROX_ReaderGetDevice(pchar(@buffer), sizeof(buffer));
      if rc = 0 then
      begin
         status.SimpleText := 'Connected, reader found on ' + StrPas(pchar(@buffer));
      end else
      begin
         status.SimpleText := 'Connected, unknown port (' + StrPas(SPROX_GetErrorMessage(rc)) + ')';
      end;

      // OK
      btnReaderClose.Enabled := true;
      cbAPolling.Enabled := true;
      cbBPolling.Enabled := true;
   end else
   begin
      // Failure
      btnReaderOpen.Enabled := true;
   end;
   Screen.Cursor := crDefault;
end;

procedure TfShowUid.btnReaderCloseClick(Sender: TObject);
var
   rc : sword;
begin
   btnReaderClose.Enabled := false;

   //
   // Disconnect from the reader
   // --------------------------
   //

   rc := SPROX_ReaderClose;
   if rc = 0 then
   begin
      // Success
      Status.SimpleText := 'Connection has been closed';
   end else
   begin
      // Failure
      Status.SimpleText := StrPas(SPROX_GetErrorMessage(rc));
   end;

   btnReaderOpen.Enabled := true;
end;

procedure TfShowUid.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   SPROX_ReaderClose;
end;

procedure TfShowUid.FormCreate(Sender: TObject);
var
   ini : TIniFile;
   pic : TPicture;
   str_company : string;
   str_logo    : string;
   str_product : string;
begin
   ini := TIniFile.Create(ExtractFileDir(Application.ExeName) + '\csbinfo.ini');
   str_company := ini.ReadString('company', 'name', 'SpringCard');
   str_logo    := ini.ReadString('company', 'logo', 'logo_pro-active.bmp');
   str_product := ini.ReadString('product', 'name', 'CSB / SpringProx');
   ini.Free;
   pic := TPicture.Create;
   try
      pic.LoadFromFile(str_logo);
      imgLogo.Left := pTop.Width - 8 - pic.Width;
      imgLogo.Picture.Assign(pic);
   except
      // No picture !
   end;
   pic.Free;
   Caption := str_company + ' ' + str_product + ' - ShowUid Sample Project';

   sgTags.Cells[0,0] := 'UID/PUPI (hex)';
   sgTags.Cells[1,0] := 'ATQA';
   sgTags.Cells[2,0] := 'SAK';
   sgTags.Cells[3,0] := 'Known type';
   sgTags.Cells[4,0] := 'UID/PUPI (bcd)';

   sgTags.ColWidths[0] := 105;
   sgTags.ColWidths[1] := 35;
   sgTags.ColWidths[2] := 25;
   sgTags.ColWidths[3] := 160;
   sgTags.ColWidths[4] := 85;

   sgTags.Row := sgTags.RowCount - 1;

   toggle := true;
end;

procedure TfShowUid.tmrPollTimer(Sender: TObject);
var
   str    : string;
   i, j   : integer;
   snr    : array[0..11] of byte;
   snrlen : byte;
   atqa   : array[0..1] of byte;
   atqb   : array[0..11] of byte;
   atq_w  : word;
   sak    : array[0..0] of byte;
   sak_b  : byte;
   dw     : int64;
   rc     : sword;
begin
   tmrPoll.Enabled := false;

   if toggle and cbAPolling.Checked then
   begin
      // Configure the reader in ISO 14443-A
      rc := SPROX_SetConfig($01);
      if rc <> 0 then
      begin
         tmrPoll.Enabled := true;
         exit;
      end;

      // Perform ISO 14443-A activate(idle)->anticoll->select sequence
      snrlen := 12;
      rc := SPROX_TclA_ActivateIdle(@atqa, @snr, snrlen, @sak);
      if (rc = 0) or (rc = -131) then
      begin
         // We have found a tag
         // - rc = 0     => OK, tag is T=CL compliant,
         // - rc = -131  => OK also, but tag is not T=CL compliant

         // Make room in the list
         for i := sgTags.RowCount - 1 downto 2 do
         begin
            for j := 0 to sgTags.ColCount - 1 do
               sgTags.Cells[j, i] := sgTags.Cells[j, i-1];
         end;

         // Display serial number
         str := '';
         if snrlen > 4 then
         begin
            // Display serial number in "natural order"
            for i := 0 to snrlen - 1 do str := str + IntToHex(snr[i], 2);
         end else
         begin
            // For tags with short serial number (Mifare mostly), display
            // the serial number using "Philips historical order"
            for i := snrlen - 1 downto 0 do str := str + IntToHex(snr[i], 2);
         end;
         sgTags.Cells[0,1] := str;

         // Display the serial number in decimal form
         dw := 0;
         for i := 0 to 3 do
         begin
            dw := dw * $00000100;
            dw := dw + snr[i];
         end;
         str := IntToStr(dw);
         sgTags.Cells[4,1] := str;

         // Display ATQ (A), 2 bytes, in "Philips historical order"
         str := '';
         for i := 1 downto 0 do str := str + IntToHex(atqa[i], 2);
         sgTags.Cells[1,1] := str;
         atq_w := atqa[0] + atqa[1] * $0100;

         // Display SAK, 1 byte
         str := IntToHex(sak[0], 2);
         sgTags.Cells[2,1] := str;
         sak_b := sak[0];

         // Try to recognize the card, see Philips' document "Mifare type identification procedure"
         str := '';
         if (atq_w = $0010) and (sak_b = $81) then str := 'Mifare Light (Philips)' else
         if (atq_w = $0044) and (sak_b = $00) then str := 'Mifare UltraLight (Philips)' else
         if (atq_w = $0004) then
         begin
            if (sak_b = $08) then str := 'Mifare 1K (Philips)' else str := 'Mifare 1K (not Philips)';
         end else
         if (atq_w = $0002) then
         begin
            if (sak_b = $18) then str := 'Mifare 4K (Philips)' else str := 'Mifare 4K (not Philips)';
         end else
         if (atq_w = $0344) and (sak_b = $20) then str := 'DesFire (Philips)' else
         begin
            if rc = 0 then
              str := 'ISO/IEC 14443-A (T=CL)'
            else
              str := 'ISO/IEC 14443-A (layer 3 only)';
         end;

         sgTags.Cells[3,1] := str;

         // Halt the tag
         SPROX_TclA_Halt;
      end;
   end;

   if (not toggle) and cbBPolling.Checked then
   begin
      // Configure the reader in ISO 14443-B
      rc := SPROX_SetConfig($02);
      if rc <> 0 then
      begin
         tmrPoll.Enabled := true;
         exit;
      end;

      // Perform ISO 14443-B reqB, AFI=0 (all cards must answer)
      snrlen := 12;
      rc := SPROX_TclB_ActivateIdle(0, @atqb);
      if (rc = 0) or (rc = -131) then
      begin
         // We have found a tag
         // - rc = 0     => OK, tag is T=CL compliant,
         // - rc = -131  => OK also, but tag is not T=CL compliant

         // Make room in the list
         for i := sgTags.RowCount - 1 downto 2 do
         begin
            for j := 0 to sgTags.ColCount - 1 do
               sgTags.Cells[j, i] := sgTags.Cells[j, i-1];
         end;

         // Display PUPI (first-4 bytes of ATQB)
         str := '';
         for i := 0 to 3 do str := str + IntToHex(atqb[i], 2);
         sgTags.Cells[0,1] := str;

         // No ATQA, no SAK
         sgTags.Cells[1,1] := '';
         sgTags.Cells[2,1] := '';

         // Type B
         if rc = 0 then
           sgTags.Cells[3,1] := 'ISO/IEC 14443-B (T=CL)'
         else
           sgTags.Cells[3,1] := 'ISO/IEC 14443-B (layer 3 only)';

         // Display the serial number in decimal form
         dw := 0;
         for i := 0 to 3 do
         begin
            dw := dw * $00000100;
            dw := dw + atqb[i];
         end;
         str := IntToStr(dw);
         sgTags.Cells[4,1] := str;

         // Halt the tag
         SPROX_TclB_Halt(@atqb);
      end;
   end;

   toggle := not toggle;
   tmrPoll.Enabled := true;
end;

end.
